@@ -4,8 +4,8 @@
 use anyhow::Result;
 use structopt::StructOpt;
 
-use openpgp_card::{Error, StatusBytes};
-use openpgp_card_pcsc::PcscClient;
+use openpgp_card::{Error, OpenPgp, StatusBytes};
+use openpgp_card_pcsc::PcscBackend;
 use openpgp_card_sequoia::card::Open;
 
 mod cli;
@@ -15,31 +15,28 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     let cli = cli::Cli::from_args();
 
-    let mut card = PcscClient::open_by_ident(&cli.ident)?;
-    let pinpad_verify = card.feature_pinpad_verify();
-    let pinpad_modify = card.feature_pinpad_modify();
+    let mut card = PcscBackend::open_by_ident(&cli.ident, None)?;
+    let mut pgp = OpenPgp::new(&mut card);
 
-    let mut open = Open::new(&mut card)?;
+    let pgpt = pgp.transaction()?;
+    let pinpad_verify = pgpt.feature_pinpad_verify();
+    let pinpad_modify = pgpt.feature_pinpad_modify();
+
+    let mut open = Open::new(pgpt)?;
 
     match cli.cmd {
         cli::Command::SetUserPin {} => {
             let res = if !pinpad_modify {
                 // get current user pin
-                let pin = rpassword::read_password_from_tty(Some(
-                    "Enter user PIN: ",
-                ))?;
+                let pin = rpassword::read_password_from_tty(Some("Enter user PIN: "))?;
 
                 // verify pin
                 open.verify_user(&pin)?;
                 println!("PIN was accepted by the card.\n");
 
                 // get new user pin
-                let newpin1 = rpassword::read_password_from_tty(Some(
-                    "Enter new user PIN: ",
-                ))?;
-                let newpin2 = rpassword::read_password_from_tty(Some(
-                    "Repeat the new user PIN: ",
-                ))?;
+                let newpin1 = rpassword::read_password_from_tty(Some("Enter new user PIN: "))?;
+                let newpin2 = rpassword::read_password_from_tty(Some("Repeat the new user PIN: "))?;
 
                 if newpin1 != newpin2 {
                     return Err(anyhow::anyhow!("PINs do not match.").into());
@@ -69,20 +66,15 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         cli::Command::SetAdminPin {} => {
             if !pinpad_modify {
                 // get current admin pin
-                let pin = rpassword::read_password_from_tty(Some(
-                    "Enter admin PIN: ",
-                ))?;
+                let pin = rpassword::read_password_from_tty(Some("Enter admin PIN: "))?;
 
                 // verify pin
                 open.verify_admin(&pin)?;
 
                 // get new admin pin
-                let newpin1 = rpassword::read_password_from_tty(Some(
-                    "Enter new admin PIN: ",
-                ))?;
-                let newpin2 = rpassword::read_password_from_tty(Some(
-                    "Repeat the new admin PIN: ",
-                ))?;
+                let newpin1 = rpassword::read_password_from_tty(Some("Enter new admin PIN: "))?;
+                let newpin2 =
+                    rpassword::read_password_from_tty(Some("Repeat the new admin PIN: "))?;
 
                 if newpin1 != newpin2 {
                     return Err(anyhow::anyhow!("PINs do not match.").into());
@@ -106,26 +98,20 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             // verify admin pin
             if !pinpad_verify {
                 // get current admin pin
-                let pin = rpassword::read_password_from_tty(Some(
-                    "Enter admin PIN: ",
-                ))?;
+                let pin = rpassword::read_password_from_tty(Some("Enter admin PIN: "))?;
 
                 open.verify_admin(&pin)?;
             } else {
-                open.verify_admin_pinpad(&|| {
-                    println!("Enter admin PIN on card reader pinpad.")
-                })?;
+                open.verify_admin_pinpad(&|| println!("Enter admin PIN on card reader pinpad."))?;
             }
             println!("PIN was accepted by the card.\n");
 
             if let Some(mut admin) = open.admin_card() {
                 // ask user for new resetting code
-                let newpin1 = rpassword::read_password_from_tty(Some(
-                    "Enter new resetting code: ",
-                ))?;
-                let newpin2 = rpassword::read_password_from_tty(Some(
-                    "Repeat the new resetting code: ",
-                ))?;
+                let newpin1 =
+                    rpassword::read_password_from_tty(Some("Enter new resetting code: "))?;
+                let newpin2 =
+                    rpassword::read_password_from_tty(Some("Repeat the new resetting code: "))?;
 
                 if newpin1 == newpin2 {
                     admin.set_resetting_code(&newpin1)?;
@@ -133,10 +119,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                     return Err(anyhow::anyhow!("PINs do not match.").into());
                 }
             } else {
-                return Err(anyhow::anyhow!(
-                    "Failed to use card in admin-mode."
-                )
-                .into());
+                return Err(anyhow::anyhow!("Failed to use card in admin-mode.").into());
             }
             println!("\nResetting code has been set.");
         }
@@ -146,9 +129,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             let rst = if admin {
                 if !pinpad_verify {
                     // get current admin pin
-                    let pin = rpassword::read_password_from_tty(Some(
-                        "Enter admin PIN: ",
-                    ))?;
+                    let pin = rpassword::read_password_from_tty(Some("Enter admin PIN: "))?;
 
                     // verify pin
                     open.verify_admin(&pin)?;
@@ -162,9 +143,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                 None
             } else {
                 // get resetting code
-                let rst = rpassword::read_password_from_tty(Some(
-                    "Enter resetting code: ",
-                ))?;
+                let rst = rpassword::read_password_from_tty(Some("Enter resetting code: "))?;
 
                 // NOTE: this code cannot be verified with the card!
 
@@ -172,12 +151,8 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             };
 
             // get new user pin
-            let newpin1 = rpassword::read_password_from_tty(Some(
-                "Enter new user PIN: ",
-            ))?;
-            let newpin2 = rpassword::read_password_from_tty(Some(
-                "Repeat the new user PIN: ",
-            ))?;
+            let newpin1 = rpassword::read_password_from_tty(Some("Enter new user PIN: "))?;
+            let newpin2 = rpassword::read_password_from_tty(Some("Repeat the new user PIN: "))?;
 
             if newpin1 != newpin2 {
                 return Err(anyhow::anyhow!("PINs do not match.").into());
@@ -189,10 +164,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             } else if let Some(mut admin) = open.admin_card() {
                 admin.reset_user_pin(&newpin1)
             } else {
-                return Err(anyhow::anyhow!(
-                    "Failed to use card in admin-mode."
-                )
-                .into());
+                return Err(anyhow::anyhow!("Failed to use card in admin-mode.").into());
             };
 
             if res.is_err() {
@@ -222,10 +194,7 @@ fn print_gnuk_note(err: Error, card: &Open) -> Result<()> {
     ) {
         // check if no keys exist on the card
         let fps = card.fingerprints()?;
-        if fps.signature() == None
-            && fps.decryption() == None
-            && fps.authentication() == None
-        {
+        if fps.signature() == None && fps.decryption() == None && fps.authentication() == None {
             println!(
                 "\nNOTE: Some cards (e.g. Gnuk) don't allow \
                         User PIN change while no keys exist on the card."
