@@ -11,12 +11,13 @@ use sequoia_openpgp::serialize::stream::{Armorer, Message, Signer};
 use sequoia_openpgp::serialize::SerializeInto;
 use sequoia_openpgp::Cert;
 
+use openpgp_card::algorithm::AlgoSimple;
+use openpgp_card::card_do::Sex;
+use openpgp_card::{CardBackend, KeyType, OpenPgp};
 use openpgp_card_sequoia::card::{Admin, Open};
 use openpgp_card_sequoia::util::{make_cert, public_key_material_to_key};
 use openpgp_card_sequoia::{sq_util, PublicKey};
 
-use openpgp_card::algorithm::AlgoSimple;
-use openpgp_card::{card_do::Sex, KeyType};
 use std::io::Write;
 
 mod cli;
@@ -55,10 +56,9 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             if detached {
                 sign_detached(&ident, user_pin, &cert_file, input.as_deref())?;
             } else {
-                return Err(anyhow::anyhow!(
-                    "Only detached signatures are supported for now"
-                )
-                .into());
+                return Err(
+                    anyhow::anyhow!("Only detached signatures are supported for now").into(),
+                );
             }
         }
         cli::Command::FactoryReset { ident } => {
@@ -70,18 +70,18 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             cmd,
         } => {
             let mut card = util::open_card(&ident)?;
-            let mut open = Open::new(&mut card)?;
+            let mut pgp = OpenPgp::new(&mut card);
+
+            let mut open = Open::new(pgp.transaction()?)?;
 
             match cmd {
                 cli::AdminCommand::Name { name } => {
-                    let mut admin =
-                        util::verify_to_admin(&mut open, admin_pin)?;
+                    let mut admin = util::verify_to_admin(&mut open, admin_pin)?;
 
                     let _ = admin.set_name(&name)?;
                 }
                 cli::AdminCommand::Url { url } => {
-                    let mut admin =
-                        util::verify_to_admin(&mut open, admin_pin)?;
+                    let mut admin = util::verify_to_admin(&mut open, admin_pin)?;
 
                     let _ = admin.set_url(&url)?;
                 }
@@ -100,9 +100,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                         // and if so, import these keys to the card.
                         key_import_yolo(admin, &key)?;
                     } else {
-                        key_import_explicit(
-                            admin, &key, sig_fp, dec_fp, auth_fp,
-                        )?;
+                        key_import_explicit(admin, &key, sig_fp, dec_fp, auth_fp)?;
                     }
                 }
                 cli::AdminCommand::Generate {
@@ -135,7 +133,8 @@ fn list_cards() -> Result<()> {
         println!("Available OpenPGP cards:");
 
         for mut card in cards {
-            let open = Open::new(&mut card)?;
+            let mut pgp = OpenPgp::new(&mut card);
+            let open = Open::new(pgp.transaction()?)?;
             println!(" {}", open.application_identifier()?.ident());
         }
     } else {
@@ -144,29 +143,32 @@ fn list_cards() -> Result<()> {
     Ok(())
 }
 
-fn set_identity(
-    ident: &str,
-    id: u8,
-) -> Result<(), Box<dyn std::error::Error>> {
+fn set_identity(ident: &str, id: u8) -> Result<(), Box<dyn std::error::Error>> {
     let mut card = util::open_card(ident)?;
+    let mut pgp = OpenPgp::new(&mut card);
 
-    card.set_identity(id)?;
+    let mut pgpt = pgp.transaction()?;
+    pgpt.set_identity(id)?;
 
     Ok(())
 }
 
 fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
-    let mut ca = if let Some(ident) = ident {
-        util::open_card(&ident)?
+    let mut card: Box<dyn CardBackend> = if let Some(ident) = ident {
+        Box::new(util::open_card(&ident)?)
     } else {
         let mut cards = util::cards()?;
         if cards.len() == 1 {
-            cards.pop().unwrap()
+            Box::new(cards.pop().unwrap())
         } else {
             return Err(anyhow::anyhow!("Found {} cards", cards.len()));
         }
     };
-    let mut open = Open::new(&mut ca)?;
+
+    let mut pgp = OpenPgp::new(&mut *card);
+    let mut open = Open::new(pgp.transaction()?)?;
+
+    let ident = open.application_identifier()?.ident();
 
     print!("OpenPGP card {}", open.application_identifier()?.ident());
 
@@ -178,6 +180,8 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     let crd = open.cardholder_related_data()?;
 
     if let Some(name) = crd.name() {
+        let name = String::from_utf8_lossy(name).to_string();
+
         print!("Cardholder: ");
 
         // This field is silly, maybe ignore it?!
@@ -202,12 +206,7 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     }
 
     if let Some(lang) = crd.lang() {
-        let lang = lang
-            .iter()
-            .map(|lang| lang.iter().collect::<String>())
-            .collect::<Vec<_>>()
-            .join(", ");
-        println!("Language preferences '{}'", lang);
+        println!("Language preferences '{:?}'", lang);
     }
 
     // information about subkeys
@@ -252,12 +251,20 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     if let Some(fp) = fps.authentication() {
         println!("  fingerprint: {}", fp.to_spaced_hex());
     }
+    let pubkey = open.public_key(KeyType::Authentication);
+    if let Ok(pkm) = &pubkey {
+        if let Ok(ssh) = util::get_ssh_pubkey_string(pkm, ident) {
+            // print auth key as openssh public key string
+            println!("  {}", ssh);
+        }
+    }
+
     if let Some(kgt) = kgt.authentication() {
         println! {"  created: {}", kgt.formatted()};
     }
     println! {"  algorithm: {}", open.algorithm_attributes(KeyType::Authentication)?};
     if verbose {
-        if let Ok(pkm) = open.public_key(KeyType::Authentication) {
+        if let Ok(pkm) = pubkey {
             println! {"  public key material: {}", pkm};
         }
     }
@@ -296,8 +303,7 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
 
         // YubiKey specific (?) firmware version
         if let Ok(ver) = open.firmware_version() {
-            let ver =
-                ver.iter().map(u8::to_string).collect::<Vec<_>>().join(".");
+            let ver = ver.iter().map(u8::to_string).collect::<Vec<_>>().join(".");
 
             println!("Firmware Version: {}", ver);
         }
@@ -318,10 +324,12 @@ fn decrypt(
     let input = util::open_or_stdin(input.as_deref())?;
 
     let mut card = util::open_card(ident)?;
-    let mut open = Open::new(&mut card)?;
+    let mut pgp = OpenPgp::new(&mut card);
+
+    let mut open = Open::new(pgp.transaction()?)?;
 
     let mut user = util::verify_to_user(&mut open, pin_file)?;
-    let d = user.decryptor(&cert, &p)?;
+    let d = user.decryptor(&cert)?;
 
     let db = DecryptorBuilder::from_reader(input)?;
     let mut decryptor = db.with_policy(&p, None, d)?;
@@ -337,16 +345,17 @@ fn sign_detached(
     cert_file: &Path,
     input: Option<&Path>,
 ) -> Result<(), Box<dyn std::error::Error>> {
-    let p = StandardPolicy::new();
     let cert = Cert::from_file(cert_file)?;
 
     let mut input = util::open_or_stdin(input.as_deref())?;
 
     let mut card = util::open_card(ident)?;
-    let mut open = Open::new(&mut card)?;
+    let mut pgp = OpenPgp::new(&mut card);
+
+    let mut open = Open::new(pgp.transaction()?)?;
 
     let mut sign = util::verify_to_sign(&mut open, pin_file)?;
-    let s = sign.signer(&cert, &p)?;
+    let s = sign.signer(&cert)?;
 
     let message = Armorer::new(Message::new(std::io::stdout())).build()?;
     let mut signer = Signer::new(message, s).detached().build()?;
@@ -360,7 +369,10 @@ fn sign_detached(
 fn factory_reset(ident: &str) -> Result<()> {
     println!("Resetting Card {}", ident);
     let mut card = util::open_card(ident)?;
-    Open::new(&mut card)?.factory_reset()
+    let mut pgp = OpenPgp::new(&mut card);
+
+    let mut open = Open::new(pgp.transaction()?)?;
+    open.factory_reset().map_err(|e| anyhow!(e))
 }
 
 fn key_import_yolo(mut admin: Admin, key: &Cert) -> Result<()> {
@@ -398,9 +410,7 @@ fn key_import_explicit(
     let p = StandardPolicy::new();
 
     if let Some(sig_fp) = sig_fp {
-        if let Some(sig) =
-            sq_util::private_subkey_by_fingerprint(key, &p, &sig_fp)?
-        {
+        if let Some(sig) = sq_util::private_subkey_by_fingerprint(key, &p, &sig_fp)? {
             println!("Uploading {} as signing key", sig.fingerprint());
             admin.upload_key(sig, KeyType::Signing, None)?;
         } else {
@@ -409,9 +419,7 @@ fn key_import_explicit(
     }
 
     if let Some(dec_fp) = dec_fp {
-        if let Some(dec) =
-            sq_util::private_subkey_by_fingerprint(key, &p, &dec_fp)?
-        {
+        if let Some(dec) = sq_util::private_subkey_by_fingerprint(key, &p, &dec_fp)? {
             println!("Uploading {} as decryption key", dec.fingerprint());
             admin.upload_key(dec, KeyType::Decryption, None)?;
         } else {
@@ -420,9 +428,7 @@ fn key_import_explicit(
     }
 
     if let Some(auth_fp) = auth_fp {
-        if let Some(auth) =
-            sq_util::private_subkey_by_fingerprint(key, &p, &auth_fp)?
-        {
+        if let Some(auth) = sq_util::private_subkey_by_fingerprint(key, &p, &auth_fp)? {
             println!("Uploading {} as authentication key", auth.fingerprint());
             admin.upload_key(auth, KeyType::Authentication, None)?;
         } else {
@@ -492,9 +498,7 @@ fn generate_keys(
                 your user PIN multiple times to make binding signatures."
             );
         } else {
-            return Err(anyhow!(
-                "No user PIN file provided, and no pinpad found"
-            ));
+            return Err(anyhow!("No user PIN file provided, and no pinpad found"));
         }
         None
     };
@@ -520,14 +524,13 @@ fn gen_subkeys(
     // We begin by generating the signing subkey, which is mandatory.
     println!(" Generate subkey for Signing");
     let (pkm, ts) = admin.generate_key_simple(KeyType::Signing, algo)?;
-    let key_sig = public_key_material_to_key(&pkm, KeyType::Decryption, ts)?;
+    let key_sig = public_key_material_to_key(&pkm, KeyType::Signing, ts)?;
 
     // make decryption subkey (unless disabled), with the same algorithm as
     // the sig key
     let key_dec = if decrypt {
         println!(" Generate subkey for Decryption");
-        let (pkm, ts) =
-            admin.generate_key_simple(KeyType::Decryption, algo)?;
+        let (pkm, ts) = admin.generate_key_simple(KeyType::Decryption, algo)?;
         Some(public_key_material_to_key(&pkm, KeyType::Decryption, ts)?)
     } else {
         None
@@ -537,8 +540,7 @@ fn gen_subkeys(
     // algorithm as the sig key
     let key_aut = if auth {
         println!(" Generate subkey for Authentication");
-        let (pkm, ts) =
-            admin.generate_key_simple(KeyType::Authentication, algo)?;
+        let (pkm, ts) = admin.generate_key_simple(KeyType::Authentication, algo)?;
 
         Some(public_key_material_to_key(
             &pkm,
